@@ -16,6 +16,7 @@
 #include <style-font.h>
 #include <style-color.h>
 #include <style-conditions.h>
+#include <sheet-conditions.h>
 #include <validation.h>
 #include <pattern.h>
 #include <hlink.h>
@@ -104,8 +105,6 @@ struct _GnmStyle {
 	GnmInputMsg		*input_msg;
 	GnmStyleConditions	*conditions;
 	GPtrArray		*cond_styles;
-
-	GPtrArray *deps;
 };
 
 #define elem_changed(style, elem) do { (style)->changed |= (1u << (elem)); } while(0)
@@ -869,12 +868,6 @@ gnm_style_unref (GnmStyle const *style)
 		gnm_style_clear_pango (unconst);
 		gnm_style_clear_font (unconst);
 
-		if (style->deps) {
-			if (style->deps->len > 0)
-				g_warning ("Leftover style deps!");
-			g_ptr_array_free (style->deps, TRUE);
-		}
-
 		CHUNK_FREE (gnm_style_pool, unconst);
 	}
 }
@@ -998,7 +991,15 @@ gnm_style_linked_sheet_changed (GnmStyle *style)
 	if (elem_is_set (style, MSTYLE_CONDITIONS) &&
 	    style->conditions &&
 	    gnm_style_conditions_get_sheet (style->conditions) != sheet) {
-		GnmStyleConditions *new_c = gnm_style_conditions_dup_to (style->conditions, sheet);
+		GnmStyleConditions *new_c, *new_sc;
+
+		sheet_conditions_share_conditions_remove (style->conditions);
+		new_c = gnm_style_conditions_dup_to (style->conditions, sheet);
+		new_sc = sheet_conditions_share_conditions_add (new_c);
+		if (new_sc) {
+			g_object_unref (new_c);
+			new_c = new_sc;
+		}
 		gnm_style_set_conditions (style, new_c);
 	}
 }
@@ -1041,6 +1042,14 @@ gnm_style_link_sheet (GnmStyle *style, Sheet *sheet)
 	style = link_border_colors (style, auto_color, style_is_orig);
 	style_color_unref (auto_color);
 
+	if (elem_is_set (style, MSTYLE_CONDITIONS) && style->conditions) {
+		// We actually change the style here, but the resulting
+		// ->conditions should be equivalent.
+		GnmStyleConditions *sc_new = sheet_conditions_share_conditions_add (style->conditions);
+		if (sc_new)
+			gnm_style_set_conditions (style, g_object_ref (sc_new));
+	}
+
 	style->linked_sheet = sheet;
 	style->link_count = 1;
 
@@ -1066,6 +1075,8 @@ gnm_style_unlink (GnmStyle *style)
 
 	d(("unlink %p = %d\n", style, style->link_count-1));
 	if (style->link_count-- == 1) {
+		if (elem_is_set (style, MSTYLE_CONDITIONS) && style->conditions)
+			sheet_conditions_share_conditions_remove (style->conditions);
 		sheet_style_unlink (style->linked_sheet, style);
 		style->linked_sheet = NULL;
 		gnm_style_unref (style);
@@ -2304,27 +2315,6 @@ gnm_style_get_cond_style (GnmStyle const *style, int ix)
 	return g_ptr_array_index (style->cond_styles, ix);
 }
 
-
-/*
- * Just a simple version for now.  We can also ignore most function
- * calls[1] and self-references[2].
- *
- * [1] Excluding volatile (TODAY, ...) and those that can create references
- * outside the arguments (INDIRECT).
- *
- * [2] References that print like A1 when used in A1.
- */
-static gboolean
-cond_expr_harmless (GnmExpr const *expr)
-{
-	GnmValue const *v = gnm_expr_get_constant (expr);
-	if (v && !VALUE_IS_CELLRANGE (v))
-		return TRUE;
-
-	return FALSE;
-}
-
-
 void
 gnm_style_link_dependents (GnmStyle *style, GnmRange const *r)
 {
@@ -2336,100 +2326,46 @@ gnm_style_link_dependents (GnmStyle *style, GnmRange const *r)
 
 	sheet = style->linked_sheet;
 
-	/*
-	 * Conditional formatting.
-	 *
-	 * We need to trigger a reformatting of the cell if a cell referenced
-	 * by the condition changes.
-	 */
+	// ----------------------------------------
+
+	// Conditional formatting.
+	//
+	// We need to trigger a reformatting of the cell if a cell referenced
+	// by the condition changes.
 	sc = elem_is_set (style, MSTYLE_CONDITIONS)
 		? gnm_style_get_conditions (style)
 		: NULL;
 	if (sc) {
-		GnmParsePos pp;
-		GPtrArray const *conds = gnm_style_conditions_details (sc);
-		guint ui;
-
-		parse_pos_init (&pp, NULL, sheet, r->start.col, r->start.row);
-
-		for (ui = 0; conds && ui < conds->len; ui++) {
-			GnmStyleCond const *c = g_ptr_array_index (conds, ui);
-			guint ei;
-
-			for (ei = 0; ei < 2; ei++) {
-				GnmExprTop const *texpr =
-					gnm_style_cond_get_expr (c, ei);
-				char *s = NULL;
-				if (!texpr)
-					continue;
-
-				if (debug_style_deps)
-					s = gnm_expr_top_as_string (texpr, &pp,
-								    sheet_get_conventions (sheet));
-
-				if (cond_expr_harmless (texpr->expr)) {
-					if (debug_style_deps) {
-						g_printerr ("Not linking %s %d:%d for %p: %s (harmless)\n",
-							    range_as_string (r), ui, ei, style, s);
-						g_free (s);
-					}
-					continue;
-				}
-
-				if (debug_style_deps) {
-					g_printerr ("Linking %s %d:%d for %p: %s\n",
-						    range_as_string (r), ui, ei, style, s);
-					g_free (s);
-				}
-
-				if (!style->deps)
-					style->deps = g_ptr_array_new ();
-				gnm_dep_style_dependency
-					(sheet, texpr, r, style->deps);
-			}
-		}
+		sheet_conditions_add (sheet, r, style);
 	}
 
-	/*
-	 * Validations.
-	 *
-	 * We can probably ignore those.  If a dependent cell changes such
-	 * that a validation condition is no longer satisfied, it is
-	 * grandfathered in as valid.
-	 */
-
-	/* The style owns the deps.  */
+	// ----------------------------------------
+	// Validations.
+	//
+	// We can probably ignore those.  If a dependent cell changes such
+	// that a validation condition is no longer satisfied, it is
+	// grandfathered in as valid.
 }
 
 void
 gnm_style_unlink_dependents (GnmStyle *style, GnmRange const *r)
 {
-	unsigned ui, k;
+	GnmStyleConditions *sc;
+	Sheet *sheet;
 
 	g_return_if_fail (style != NULL);
 	g_return_if_fail (r != NULL);
 
-	if (!style->deps)
-		return;
+	sheet = style->linked_sheet;
 
-	for (ui = k = 0; ui < style->deps->len; ui++) {
-		GnmDependent *dep = g_ptr_array_index (style->deps, ui);
-		GnmCellPos const *pos = dependent_pos (dep);
-
-		if (dep->sheet->being_destructed ||
-		    range_contains (r, pos->col, pos->row)) {
-			if (debug_style_deps)
-				g_printerr ("Unlinking %s for %p\n",
-					    cellpos_as_string (pos), style);
-			dependent_set_expr (dep, NULL);
-			g_free (dep);
-		} else {
-			g_ptr_array_index (style->deps, k) = dep;
-			k++;
-		}
+	sc = elem_is_set (style, MSTYLE_CONDITIONS)
+		? gnm_style_get_conditions (style)
+		: NULL;
+	if (sc) {
+		sheet_conditions_remove (sheet, r, style);
 	}
 
-	g_ptr_array_set_size (style->deps, k);
+	// Validation -- see gnm_style_link_dependents
 }
 
 
