@@ -837,7 +837,7 @@ run_tool_test (const char *tool, char **argv, WorkbookView *wbv)
 		data->base.range_2 = GET_ARG (RANGE_ARG, "y", value_new_error_REF (NULL));
 		data->base.labels = GET_ARG (atoi, "labels", FALSE);
 		data->base.alpha = GET_ARG (atof, "alpha", 0.05);
-		data->group_by = GET_ARG ((group_by_t), "grouped-by", GROUPED_BY_COL);
+		data->group_by = GET_ARG ((group_by_t)atoi, "grouped-by", GROUPED_BY_COL);
 		data->intercept = GET_ARG (atoi, "intercept", TRUE);
 		data->multiple_regression = GET_ARG (atoi, "multiple", TRUE);
 		data->multiple_y = GET_ARG (atoi, "multiple-y", FALSE);
@@ -851,7 +851,7 @@ run_tool_test (const char *tool, char **argv, WorkbookView *wbv)
 
 		data->base.input = GET_ARG (RANGE_LIST_ARG, "data", NULL);
 		data->base.labels = GET_ARG (atoi, "labels", FALSE);
-		data->base.group_by = GET_ARG ((group_by_t), "grouped-by", GROUPED_BY_COL);
+		data->base.group_by = GET_ARG ((group_by_t)atoi, "grouped-by", GROUPED_BY_COL);
 		data->alpha = GET_ARG (atof, "alpha", 0.05);
 
 		engine = analysis_tool_anova_single_engine;
@@ -1125,12 +1125,13 @@ convert (char const *inarg, char const *outarg, char const *mergeargs[],
 	GOFileOpener *fo = NULL;
 	char *infile = go_shell_arg_to_uri (inarg);
 	char *outfile = outarg ? go_shell_arg_to_uri (outarg) : NULL;
-	WorkbookView *wbv;
+	WorkbookView *wbv = NULL;
 	GOIOContext *io_context = NULL;
 	Workbook *wb = NULL;
 	GOFileSaveScope fsscope;
 	GPtrArray *sheet_sel = NULL;
 	GnmRangeRef const *range = NULL;
+	gboolean user_selected_sheets;
 
 	if (ssconvert_object_export) {
 		if (ssconvert_export_id)
@@ -1302,8 +1303,11 @@ convert (char const *inarg, char const *outarg, char const *mergeargs[],
 				     wb,
 				     ssconvert_range);
 
+	user_selected_sheets =
+		!!g_object_get_data (G_OBJECT (wb), SSCONVERT_SHEET_SET_KEY);
+
 	if (ssconvert_one_file_per_sheet ||
-	    fsscope == GO_FILE_SAVE_SHEET ||
+	    (fsscope == GO_FILE_SAVE_SHEET && !user_selected_sheets) ||
 	    range) {
 		Sheet *def_sheet = NULL;
 
@@ -1332,6 +1336,8 @@ convert (char const *inarg, char const *outarg, char const *mergeargs[],
 	}
 
  out:
+	if (wbv)
+		wb = wb_view_get_workbook (wbv);
 	if (wb)
 		g_object_unref (wb);
 	if (io_context)
