@@ -129,8 +129,9 @@ static void
 workbook_dispose (GObject *wb_object)
 {
 	Workbook *wb = WORKBOOK (wb_object);
-	GSList *sheets, *ptr;
 	GSList *controls = NULL;
+	GPtrArray *sheets;
+	unsigned ui;
 
 	wb->during_destruction = TRUE;
 
@@ -163,11 +164,13 @@ workbook_dispose (GObject *wb_object)
 	dependents_workbook_destroy (wb);
 
 	/* Copy the set of sheets, the list changes under us. */
-	sheets = workbook_sheets (wb);
+	sheets = g_ptr_array_sized_new (wb->sheets->len);
+	for (ui = 0; ui < wb->sheets->len; ui++)
+		g_ptr_array_add (sheets, g_ptr_array_index (wb->sheets, ui));
 
 	/* Remove all contents while all sheets still exist */
-	for (ptr = sheets; ptr != NULL ; ptr = ptr->next) {
-		Sheet *sheet = ptr->data;
+	for (ui = 0; ui < sheets->len; ui++) {
+		Sheet *sheet = g_ptr_array_index (sheets, ui);
 		GnmRange r;
 
 		sheet->being_destructed = TRUE;
@@ -181,11 +184,11 @@ workbook_dispose (GObject *wb_object)
 	}
 
 	/* Now remove the sheets themselves */
-	for (ptr = sheets; ptr != NULL ; ptr = ptr->next) {
-		Sheet *sheet = ptr->data;
+	for (ui = 0; ui < sheets->len; ui++) {
+		Sheet *sheet = g_ptr_array_index (sheets, ui);
 		workbook_sheet_delete (sheet);
 	}
-	g_slist_free (sheets);
+	g_ptr_array_unref (sheets);
 
 	// Now get rid of the control refs
 	g_slist_free_full (controls, g_object_unref);
@@ -460,11 +463,19 @@ workbook_new_with_sheets (int sheet_count)
 		gnm_sheet_suggest_size (&cols, &rows);
 	while (sheet_count-- > 0)
 		workbook_sheet_add (wb, -1, cols, rows);
-	go_doc_set_dirty (GO_DOC (wb), FALSE);
+	// Restore to pristine state
+	go_doc_set_state (GO_DOC (wb), go_doc_get_saved_state (GO_DOC (wb)));
 	go_doc_set_pristine (GO_DOC (wb), TRUE);
 	return wb;
 }
 
+void
+workbook_mark_dirty (Workbook *wb)
+{
+	go_doc_bump_state (GO_DOC (wb));
+}
+
+
 /**
  * workbook_set_saveinfo:
  * @wb: the workbook to modify
@@ -829,21 +840,11 @@ workbook_detach_view (WorkbookView *wbv)
  *
  * Returns: (element-type Sheet) (transfer container): the sheets list.
  */
-GSList *
+GPtrArray *
 workbook_sheets (Workbook const *wb)
 {
-	GSList *list = NULL;
-
 	g_return_val_if_fail (GNM_IS_WORKBOOK (wb), NULL);
-
-	if (wb->sheets) {
-		int i = wb->sheets->len;
-		while (i-- > 0)
-			list = g_slist_prepend (list,
-				g_ptr_array_index (wb->sheets, i));
-	}
-
-	return list;
+	return g_ptr_array_ref (wb->sheets);
 }
 
 int
@@ -1038,7 +1039,7 @@ workbook_sheet_attach_at_pos (Workbook *wb, Sheet *new_sheet, int pos)
 	/* Do not signal until after adding the views [#314208] */
 	post_sheet_index_change (wb);
 
-	go_doc_set_dirty (GO_DOC (wb), TRUE);
+	workbook_mark_dirty (wb);
 }
 
 /**
@@ -1166,7 +1167,7 @@ workbook_sheet_delete (Sheet *sheet)
 	g_object_unref (sheet);
 
 	if (!wb->during_destruction)
-		go_doc_set_dirty (GO_DOC (wb), TRUE);
+		workbook_mark_dirty (wb);
 	g_signal_emit (G_OBJECT (wb), signals[SHEET_DELETED], 0);
 
 	if (!wb->during_destruction)
@@ -1211,7 +1212,7 @@ workbook_sheet_move (Sheet *sheet, int direction)
 
 	post_sheet_index_change (wb);
 
-	go_doc_set_dirty (GO_DOC (wb), TRUE);
+	workbook_mark_dirty (wb);
 }
 
 /**
